// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/origin_trials.cc.tmpl
// and input files:
//   ../../../../../src/3rdparty/chromium/third_party/blink/renderer/platform/runtime_enabled_features.json5


#include "third_party/blink/public/common/origin_trials/origin_trials.h"

#include <array>
#include <iterator>

#include "base/containers/contains.h"
#include "base/ranges/algorithm.h"
#include "build/build_config.h"
#include "build/buildflag.h"
#include "build/chromeos_buildflags.h"
#include "third_party/blink/public/common/origin_trials/origin_trial_feature.h"

// For testing. See OriginTrialsSampleAPIInvalidOS.
#define BUILDFLAG_INTERNAL_IS_INVALID() (0)

namespace blink {

namespace {

static constexpr size_t kMaxFeaturesPerTrial = 9;
static constexpr struct TrialToFeature {
  const char* trial_name;
  unsigned feature_count;
  std::array<OriginTrialFeature, kMaxFeaturesPerTrial> features;
} kTrialToFeaturesMap[] = {
    { "AddIdentityInCanMakePaymentEvent", 1, {OriginTrialFeature::kAddIdentityInCanMakePaymentEvent, } },
    { "AdInterestGroupAPI", 1, {OriginTrialFeature::kAdInterestGroupAPI, } },
    { "PrivacySandboxAdsAPIs", 9, {OriginTrialFeature::kAttributionReporting,OriginTrialFeature::kFencedFrames,OriginTrialFeature::kFencedFramesAPIChanges,OriginTrialFeature::kFledge,OriginTrialFeature::kPrivacySandboxAdsAPIs,OriginTrialFeature::kSharedStorageAPI,OriginTrialFeature::kTopicsAPI,OriginTrialFeature::kTopicsDocumentAPI,OriginTrialFeature::kTopicsXHR, } },
    { "AttributionReportingCrossAppWeb", 1, {OriginTrialFeature::kAttributionReportingCrossAppWeb, } },
    { "AttributionReportingInterface", 1, {OriginTrialFeature::kAttributionReportingInterface, } },
    { "AutoDarkMode", 1, {OriginTrialFeature::kAutoDarkMode, } },
    { "BackForwardCacheExperimentHTTPHeader", 1, {OriginTrialFeature::kBackForwardCacheExperimentHTTPHeader, } },
    { "BackForwardCacheNotRestoredReasons", 1, {OriginTrialFeature::kBackForwardCacheNotRestoredReasons, } },
    { "BeforeMatchEvent", 1, {OriginTrialFeature::kBeforeMatchEvent, } },
    { "CacheStorageCodeCacheHint", 1, {OriginTrialFeature::kCacheStorageCodeCacheHint, } },
    { "CompressionDictionaryTransport", 1, {OriginTrialFeature::kCompressionDictionaryTransport, } },
    { "ComputePressure_v2", 1, {OriginTrialFeature::kComputePressure, } },
    { "CoopRestrictProperties", 1, {OriginTrialFeature::kCoopRestrictProperties, } },
    { "WebSQL", 1, {OriginTrialFeature::kDatabase, } },
    { "DeviceAttributes", 1, {OriginTrialFeature::kDeviceAttributes, } },
    { "DigitalGoodsV2", 1, {OriginTrialFeature::kDigitalGoods, } },
    { "DisableDifferentOriginSubframeDialogSuppression", 1, {OriginTrialFeature::kDisableDifferentOriginSubframeDialogSuppression, } },
    { "DisableHardwareNoiseSuppression", 1, {OriginTrialFeature::kDisableHardwareNoiseSuppression, } },
    { "DisableThirdPartySessionStoragePartitioningAfterGeneralPartitioning", 1, {OriginTrialFeature::kDisableThirdPartySessionStoragePartitioningAfterGeneralPartitioning, } },
    { "DisableThirdPartyStoragePartitioning", 1, {OriginTrialFeature::kDisableThirdPartyStoragePartitioning, } },
    { "DocumentPolicyNegotiation", 1, {OriginTrialFeature::kDocumentPolicyNegotiation, } },
    { "EarlyHintsPreloadForNavigation", 1, {OriginTrialFeature::kEarlyHintsPreloadForNavigationOptIn, } },
    { "EditContext", 1, {OriginTrialFeature::kEditContext, } },
    { "FedCmIdpSigninStatus", 1, {OriginTrialFeature::kFedCmIdpSigninStatus, } },
    { "FledgeBiddingAndAuctionServer", 1, {OriginTrialFeature::kFledgeBiddingAndAuctionServerAPI, } },
    { "Focusgroup", 1, {OriginTrialFeature::kFocusgroup, } },
    { "GetAllScreensMedia", 1, {OriginTrialFeature::kGetAllScreensMedia, } },
    { "HrefTranslate", 1, {OriginTrialFeature::kHrefTranslate, } },
    { "HTMLPopupAttribute", 1, {OriginTrialFeature::kHTMLPopoverAttribute, } },
    { "IgnoreCSPInWebPaymentAPI", 1, {OriginTrialFeature::kIgnoreCSPInWebPaymentAPI, } },
    { "JavaScriptCompileHintsMagic", 1, {OriginTrialFeature::kJavaScriptCompileHintsMagicRuntime, } },
    { "LongAnimationFrameTiming", 2, {OriginTrialFeature::kLongAnimationFrameMonitoring,OriginTrialFeature::kLongAnimationFrameTiming, } },
    { "MediaCaptureBackgroundBlur", 2, {OriginTrialFeature::kMediaCaptureBackgroundBlur,OriginTrialFeature::kMediaCaptureConfigurationChange, } },
    { "MediaSourceExtensionsForWebCodecs", 1, {OriginTrialFeature::kMediaSourceExtensionsForWebCodecs, } },
    { "SoftNavigationHeuristics", 2, {OriginTrialFeature::kNavigationId,OriginTrialFeature::kSoftNavigationHeuristics, } },
    { "NotificationTriggers", 1, {OriginTrialFeature::kNotificationTriggers, } },
    { "NoVarySearchPrefetch", 2, {OriginTrialFeature::kNoVarySearchPrefetch,OriginTrialFeature::kSpeculationRulesNoVarySearchHint, } },
    { "Frobulate", 1, {OriginTrialFeature::kOriginTrialsSampleAPI, } },
    { "FrobulateBrowserReadWrite", 1, {OriginTrialFeature::kOriginTrialsSampleAPIBrowserReadWrite, } },
    { "FrobulateDeprecation", 1, {OriginTrialFeature::kOriginTrialsSampleAPIDeprecation, } },
    { "FrobulateExpiryGracePeriod", 1, {OriginTrialFeature::kOriginTrialsSampleAPIExpiryGracePeriod, } },
    { "FrobulateExpiryGracePeriodThirdParty", 1, {OriginTrialFeature::kOriginTrialsSampleAPIExpiryGracePeriodThirdParty, } },
    { "FrobulateImplied", 1, {OriginTrialFeature::kOriginTrialsSampleAPIImplied, } },
    { "FrobulateInvalidOS", 1, {OriginTrialFeature::kOriginTrialsSampleAPIInvalidOS, } },
    { "FrobulateNavigation", 1, {OriginTrialFeature::kOriginTrialsSampleAPINavigation, } },
    { "FrobulatePersistentExpiryGracePeriod", 1, {OriginTrialFeature::kOriginTrialsSampleAPIPersistentExpiryGracePeriod, } },
    { "FrobulatePersistent", 1, {OriginTrialFeature::kOriginTrialsSampleAPIPersistentFeature, } },
    { "FrobulatePersistentInvalidOS", 1, {OriginTrialFeature::kOriginTrialsSampleAPIPersistentInvalidOS, } },
    { "FrobulatePersistentThirdPartyDeprecation", 1, {OriginTrialFeature::kOriginTrialsSampleAPIPersistentThirdPartyDeprecationFeature, } },
    { "FrobulateThirdParty", 1, {OriginTrialFeature::kOriginTrialsSampleAPIThirdParty, } },
    { "PageFreezeOptIn", 1, {OriginTrialFeature::kPageFreezeOptIn, } },
    { "PageFreezeOptOut", 1, {OriginTrialFeature::kPageFreezeOptOut, } },
    { "Parakeet", 1, {OriginTrialFeature::kParakeet, } },
    { "PartitionedCookies", 1, {OriginTrialFeature::kPartitionedCookies, } },
    { "PaymentHandlerMinimalHeaderUX", 1, {OriginTrialFeature::kPaymentHandlerMinimalHeaderUX, } },
    { "PendingBeaconAPI", 1, {OriginTrialFeature::kPendingBeaconAPI, } },
    { "PerMethodCanMakePaymentQuota", 1, {OriginTrialFeature::kPerMethodCanMakePaymentQuota, } },
    { "PermissionsPolicyUnload", 1, {OriginTrialFeature::kPermissionsPolicyUnload, } },
    { "PNaCl", 1, {OriginTrialFeature::kPNaCl, } },
    { "Portals", 1, {OriginTrialFeature::kPortals, } },
    { "PrivateNetworkAccessNonSecureContextsAllowed", 1, {OriginTrialFeature::kPrivateNetworkAccessNonSecureContextsAllowed, } },
    { "TrustTokens", 1, {OriginTrialFeature::kPrivateStateTokens, } },
    { "ReduceAcceptLanguage", 1, {OriginTrialFeature::kReduceAcceptLanguage, } },
    { "RtcAudioJitterBufferMaxPackets", 1, {OriginTrialFeature::kRtcAudioJitterBufferMaxPackets, } },
    { "RTCEncodedFrameSetMetadata", 1, {OriginTrialFeature::kRTCEncodedFrameSetMetadata, } },
    { "RTCLegacyCallbackBasedGetStats", 1, {OriginTrialFeature::kRTCLegacyCallbackBasedGetStats, } },
    { "RTCStatsRelativePacketArrivalDelay", 1, {OriginTrialFeature::kRTCStatsRelativePacketArrivalDelay, } },
    { "SchedulerYield", 1, {OriginTrialFeature::kSchedulerYield, } },
    { "SecurePaymentConfirmationOptOut", 1, {OriginTrialFeature::kSecurePaymentConfirmationOptOut, } },
    { "ServiceWorkerBypassFetchHandlerForMainResource", 1, {OriginTrialFeature::kServiceWorkerBypassFetchHandler, } },
    { "ServiceWorkerBypassFetchHandlerWithRaceNetworkRequest", 1, {OriginTrialFeature::kServiceWorkerRaceNetworkRequest, } },
    { "ServiceWorkerStaticRouter", 1, {OriginTrialFeature::kServiceWorkerStaticRouter, } },
    { "SignatureBasedIntegrity", 1, {OriginTrialFeature::kSignatureBasedIntegrity, } },
    { "SpeculationRules__DONOTUSE", 1, {OriginTrialFeature::kSpeculationRules, } },
    { "SpeculationRulesPrefetchFuture", 6, {OriginTrialFeature::kSpeculationRulesDocumentRules,OriginTrialFeature::kSpeculationRulesDocumentRulesSelectorMatches,OriginTrialFeature::kSpeculationRulesEagerness,OriginTrialFeature::kSpeculationRulesFetchFromHeader,OriginTrialFeature::kSpeculationRulesPrefetchFuture,OriginTrialFeature::kSpeculationRulesRelativeToDocument, } },
    { "SpeculationRulesPrefetch", 1, {OriginTrialFeature::kSpeculationRulesPrefetchProxy, } },
    { "StorageBuckets", 1, {OriginTrialFeature::kStorageBuckets, } },
    { "TextFragmentIdentifiers", 1, {OriginTrialFeature::kTextFragmentIdentifiers, } },
    { "ForceTouchEventFeatureDetectionForInspector", 1, {OriginTrialFeature::kTouchEventFeatureDetection, } },
    { "UnrestrictedSharedArrayBuffer", 1, {OriginTrialFeature::kUnrestrictedSharedArrayBuffer, } },
    { "WebAppDarkModeV2", 1, {OriginTrialFeature::kWebAppDarkMode, } },
    { "Launch Handler", 1, {OriginTrialFeature::kWebAppLaunchHandler, } },
    { "WebAppLaunchQueue", 1, {OriginTrialFeature::kWebAppLaunchQueue, } },
    { "WebAppTabStrip", 2, {OriginTrialFeature::kWebAppTabStrip,OriginTrialFeature::kWebAppTabStripCustomizations, } },
    { "WebAppUrlHandling", 1, {OriginTrialFeature::kWebAppUrlHandling, } },
    { "WebAppWindowControlsOverlay", 1, {OriginTrialFeature::kWebAppWindowControlsOverlay, } },
    { "WebAssemblyGC", 1, {OriginTrialFeature::kWebAssemblyGC, } },
    { "WebCodecs", 1, {OriginTrialFeature::kWebCodecs, } },
    { "WebEnvironmentIntegrity", 1, {OriginTrialFeature::kWebEnvironmentIntegrity, } },
    { "WebTransportCustomCertificates", 1, {OriginTrialFeature::kWebTransportCustomCertificates, } },
    { "WebViewXRequestedWithDeprecation", 1, {OriginTrialFeature::kWebViewXRequestedWithDeprecation, } },
    { "WebXRImageTracking", 1, {OriginTrialFeature::kWebXRImageTracking, } },
    { "WebXRPlaneDetection", 1, {OriginTrialFeature::kWebXRPlaneDetection, } },
    // For testing
    { "This trial does not exist", 1, { OriginTrialFeature::kNonExisting } },
};

} // namespace

bool origin_trials::IsTrialValid(base::StringPiece trial_name) {
  return base::Contains(kTrialToFeaturesMap, trial_name,
                        &TrialToFeature::trial_name);
}

bool origin_trials::IsTrialEnabledForInsecureContext(base::StringPiece trial_name) {
  static const char* const kEnabledForInsecureContext[] = {
      "DisableDifferentOriginSubframeDialogSuppression",
      "DisableThirdPartySessionStoragePartitioningAfterGeneralPartitioning",
      "DisableThirdPartyStoragePartitioning",
      "FrobulateDeprecation",
      "PrivateNetworkAccessNonSecureContextsAllowed",
      "WebViewXRequestedWithDeprecation",
  };
  return base::Contains(kEnabledForInsecureContext, trial_name);
}

bool origin_trials::IsTrialEnabledForThirdPartyOrigins(base::StringPiece trial_name) {
  static const char* const kEnabledForThirdPartyOrigins[] = {
      "AddIdentityInCanMakePaymentEvent",
      "PrivacySandboxAdsAPIs",
      "AttributionReportingCrossAppWeb",
      "AttributionReportingInterface",
      "CompressionDictionaryTransport",
      "DeviceAttributes",
      "DisableThirdPartyStoragePartitioning",
      "FedCmIdpSigninStatus",
      "FledgeBiddingAndAuctionServer",
      "IgnoreCSPInWebPaymentAPI",
      "LongAnimationFrameTiming",
      "FrobulateExpiryGracePeriodThirdParty",
      "FrobulatePersistent",
      "FrobulatePersistentInvalidOS",
      "FrobulatePersistentThirdPartyDeprecation",
      "FrobulateThirdParty",
      "PaymentHandlerMinimalHeaderUX",
      "PendingBeaconAPI",
      "TrustTokens",
      "SecurePaymentConfirmationOptOut",
      "SoftNavigationHeuristics",
      "SpeculationRulesPrefetchFuture",
      "WebEnvironmentIntegrity",
      "WebViewXRequestedWithDeprecation",
  };
  return base::Contains(kEnabledForThirdPartyOrigins, trial_name);
}

bool origin_trials::IsTrialEnabledForBrowserProcessReadAccess(base::StringPiece trial_name) {
  // Select all features that represent origin trials and have
  // browser_process_read_write_access enabled. Determine if that list of
  // features contains the  `trial_name` provided.
  static const char* const kEnabledForBrowserProcessReadWriteAccess[] = {
      "DisableThirdPartyStoragePartitioning",
      "FedCmIdpSigninStatus",
      "FrobulateBrowserReadWrite",
  };
  return base::Contains(kEnabledForBrowserProcessReadWriteAccess, trial_name);
}

bool origin_trials::IsDeprecationTrial(base::StringPiece trial_name) {
  for (auto feature : FeaturesForTrial(trial_name)) {
    if (GetTrialType(feature) == OriginTrialType::kDeprecation) {
      return true;
    }
  }
  return false;
}

OriginTrialType origin_trials::GetTrialType(OriginTrialFeature feature) {
  switch (feature) {
    case OriginTrialFeature::kDisableDifferentOriginSubframeDialogSuppression:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kDisableThirdPartySessionStoragePartitioningAfterGeneralPartitioning:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kDisableThirdPartyStoragePartitioning:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kOriginTrialsSampleAPIDeprecation:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kOriginTrialsSampleAPIPersistentThirdPartyDeprecationFeature:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kPrivateNetworkAccessNonSecureContextsAllowed:
      return OriginTrialType::kDeprecation;
    case OriginTrialFeature::kWebViewXRequestedWithDeprecation:
      return OriginTrialType::kDeprecation;
    default:
      return OriginTrialType::kDefault;
  }
}


base::span<const OriginTrialFeature> origin_trials::FeaturesForTrial(
    base::StringPiece trial_name) {
  auto it = base::ranges::find(kTrialToFeaturesMap, trial_name,
                               &TrialToFeature::trial_name);
  DCHECK(it != std::end(kTrialToFeaturesMap));
  return {it->features.data(), it->feature_count};
}

base::span<const OriginTrialFeature> origin_trials::GetImpliedFeatures(
    OriginTrialFeature feature) {
  if (feature == OriginTrialFeature::kFledge) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kAdInterestGroupAPI,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kParakeet) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kAdInterestGroupAPI,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kAttributionReporting) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kAttributionReportingInterface,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kAttributionReportingCrossAppWeb) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kAttributionReportingInterface,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kLongAnimationFrameTiming) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kLongAnimationFrameMonitoring,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kMediaCaptureBackgroundBlur) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kMediaCaptureConfigurationChange,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kOriginTrialsSampleAPI) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kOriginTrialsSampleAPIImplied,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kOriginTrialsSampleAPIInvalidOS) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kOriginTrialsSampleAPIImplied,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kSpeculationRulesPrefetchProxy) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRules,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kSpeculationRulesDocumentRules) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRules,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kSpeculationRulesRelativeToDocument) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRules,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kSpeculationRulesNoVarySearchHint) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRulesEagerness,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kSpeculationRulesPrefetchFuture) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRulesPrefetchProxy,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kNoVarySearchPrefetch) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kSpeculationRulesPrefetchProxy,};
    return implied_features;
  }
  if (feature == OriginTrialFeature::kWebAppLaunchHandler) {
    static constexpr OriginTrialFeature implied_features[] = {OriginTrialFeature::kWebAppLaunchQueue,};
    return implied_features;
  }
  return {};
}

bool origin_trials::FeatureEnabledForOS(OriginTrialFeature feature) {
  switch (feature) {
    case OriginTrialFeature::kAddIdentityInCanMakePaymentEvent:
      return true;
    case OriginTrialFeature::kAdInterestGroupAPI:
      return true;
    case OriginTrialFeature::kAttributionReporting:
      return true;
    case OriginTrialFeature::kAttributionReportingCrossAppWeb:
      return true;
    case OriginTrialFeature::kAttributionReportingInterface:
      return true;
    case OriginTrialFeature::kAutoDarkMode:
      return true;
    case OriginTrialFeature::kBackForwardCacheExperimentHTTPHeader:
      return true;
    case OriginTrialFeature::kBackForwardCacheNotRestoredReasons:
      return true;
    case OriginTrialFeature::kBeforeMatchEvent:
      return true;
    case OriginTrialFeature::kCacheStorageCodeCacheHint:
      return true;
    case OriginTrialFeature::kCompressionDictionaryTransport:
      return true;
    case OriginTrialFeature::kComputePressure:
      return true;
    case OriginTrialFeature::kCoopRestrictProperties:
      return true;
    case OriginTrialFeature::kDatabase:
      return true;
    case OriginTrialFeature::kDeviceAttributes:
#if BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kDigitalGoods:
#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kDisableDifferentOriginSubframeDialogSuppression:
      return true;
    case OriginTrialFeature::kDisableHardwareNoiseSuppression:
      return true;
    case OriginTrialFeature::kDisableThirdPartySessionStoragePartitioningAfterGeneralPartitioning:
      return true;
    case OriginTrialFeature::kDisableThirdPartyStoragePartitioning:
      return true;
    case OriginTrialFeature::kDocumentPolicyNegotiation:
      return true;
    case OriginTrialFeature::kEarlyHintsPreloadForNavigationOptIn:
      return true;
    case OriginTrialFeature::kEditContext:
      return true;
    case OriginTrialFeature::kFedCmIdpSigninStatus:
      return true;
    case OriginTrialFeature::kFencedFrames:
      return true;
    case OriginTrialFeature::kFencedFramesAPIChanges:
      return true;
    case OriginTrialFeature::kFledge:
      return true;
    case OriginTrialFeature::kFledgeBiddingAndAuctionServerAPI:
      return true;
    case OriginTrialFeature::kFocusgroup:
      return true;
    case OriginTrialFeature::kGetAllScreensMedia:
#if BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kHrefTranslate:
      return true;
    case OriginTrialFeature::kHTMLPopoverAttribute:
      return true;
    case OriginTrialFeature::kIgnoreCSPInWebPaymentAPI:
      return true;
    case OriginTrialFeature::kJavaScriptCompileHintsMagicRuntime:
      return true;
    case OriginTrialFeature::kLongAnimationFrameMonitoring:
      return true;
    case OriginTrialFeature::kLongAnimationFrameTiming:
      return true;
    case OriginTrialFeature::kMediaCaptureBackgroundBlur:
      return true;
    case OriginTrialFeature::kMediaCaptureConfigurationChange:
      return true;
    case OriginTrialFeature::kMediaSourceExtensionsForWebCodecs:
      return true;
    case OriginTrialFeature::kNavigationId:
      return true;
    case OriginTrialFeature::kNotificationTriggers:
      return true;
    case OriginTrialFeature::kNoVarySearchPrefetch:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPI:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIBrowserReadWrite:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIDeprecation:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIExpiryGracePeriod:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIExpiryGracePeriodThirdParty:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIImplied:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIInvalidOS:
#if BUILDFLAG(IS_INVALID)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kOriginTrialsSampleAPINavigation:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIPersistentExpiryGracePeriod:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIPersistentFeature:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIPersistentInvalidOS:
#if BUILDFLAG(IS_INVALID)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kOriginTrialsSampleAPIPersistentThirdPartyDeprecationFeature:
      return true;
    case OriginTrialFeature::kOriginTrialsSampleAPIThirdParty:
      return true;
    case OriginTrialFeature::kPageFreezeOptIn:
      return true;
    case OriginTrialFeature::kPageFreezeOptOut:
      return true;
    case OriginTrialFeature::kParakeet:
      return true;
    case OriginTrialFeature::kPartitionedCookies:
      return true;
    case OriginTrialFeature::kPaymentHandlerMinimalHeaderUX:
#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_FUCHSIA) || BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kPendingBeaconAPI:
      return true;
    case OriginTrialFeature::kPerMethodCanMakePaymentQuota:
      return true;
    case OriginTrialFeature::kPermissionsPolicyUnload:
      return true;
    case OriginTrialFeature::kPNaCl:
      return true;
    case OriginTrialFeature::kPortals:
#if BUILDFLAG(IS_ANDROID)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kPrivacySandboxAdsAPIs:
      return true;
    case OriginTrialFeature::kPrivateNetworkAccessNonSecureContextsAllowed:
      return true;
    case OriginTrialFeature::kPrivateStateTokens:
      return true;
    case OriginTrialFeature::kReduceAcceptLanguage:
      return true;
    case OriginTrialFeature::kRtcAudioJitterBufferMaxPackets:
      return true;
    case OriginTrialFeature::kRTCEncodedFrameSetMetadata:
      return true;
    case OriginTrialFeature::kRTCLegacyCallbackBasedGetStats:
      return true;
    case OriginTrialFeature::kRTCStatsRelativePacketArrivalDelay:
      return true;
    case OriginTrialFeature::kSchedulerYield:
      return true;
    case OriginTrialFeature::kSecurePaymentConfirmationOptOut:
      return true;
    case OriginTrialFeature::kServiceWorkerBypassFetchHandler:
      return true;
    case OriginTrialFeature::kServiceWorkerRaceNetworkRequest:
      return true;
    case OriginTrialFeature::kServiceWorkerStaticRouter:
      return true;
    case OriginTrialFeature::kSharedStorageAPI:
      return true;
    case OriginTrialFeature::kSignatureBasedIntegrity:
      return true;
    case OriginTrialFeature::kSoftNavigationHeuristics:
      return true;
    case OriginTrialFeature::kSpeculationRules:
      return true;
    case OriginTrialFeature::kSpeculationRulesDocumentRules:
      return true;
    case OriginTrialFeature::kSpeculationRulesDocumentRulesSelectorMatches:
      return true;
    case OriginTrialFeature::kSpeculationRulesEagerness:
      return true;
    case OriginTrialFeature::kSpeculationRulesFetchFromHeader:
      return true;
    case OriginTrialFeature::kSpeculationRulesNoVarySearchHint:
      return true;
    case OriginTrialFeature::kSpeculationRulesPrefetchFuture:
      return true;
    case OriginTrialFeature::kSpeculationRulesPrefetchProxy:
      return true;
    case OriginTrialFeature::kSpeculationRulesRelativeToDocument:
      return true;
    case OriginTrialFeature::kStorageBuckets:
      return true;
    case OriginTrialFeature::kTextFragmentIdentifiers:
      return true;
    case OriginTrialFeature::kTopicsAPI:
      return true;
    case OriginTrialFeature::kTopicsDocumentAPI:
      return true;
    case OriginTrialFeature::kTopicsXHR:
      return true;
    case OriginTrialFeature::kTouchEventFeatureDetection:
      return true;
    case OriginTrialFeature::kUnrestrictedSharedArrayBuffer:
#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_FUCHSIA) || BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebAppDarkMode:
      return true;
    case OriginTrialFeature::kWebAppLaunchHandler:
      return true;
    case OriginTrialFeature::kWebAppLaunchQueue:
      return true;
    case OriginTrialFeature::kWebAppTabStrip:
#if BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebAppTabStripCustomizations:
#if BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebAppUrlHandling:
#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebAppWindowControlsOverlay:
#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebAssemblyGC:
      return true;
    case OriginTrialFeature::kWebCodecs:
      return true;
    case OriginTrialFeature::kWebEnvironmentIntegrity:
      return true;
    case OriginTrialFeature::kWebTransportCustomCertificates:
      return true;
    case OriginTrialFeature::kWebViewXRequestedWithDeprecation:
#if BUILDFLAG(IS_ANDROID)
      return true;
#else
      return false;
#endif
    case OriginTrialFeature::kWebXRImageTracking:
      return true;
    case OriginTrialFeature::kWebXRPlaneDetection:
      return true;
    // For testing
    case OriginTrialFeature::kNonExisting:
      return true;
  }
}

} // namespace blink
